;
;	read in a space delimited database of unknown column 
;	size or unknown number of rows, located in the FILE
;	and contained in directory DIR.
;	Depending upon DBTYPE, the program loads the data and
;	returns the data, DB, as array of NCOL, NROW size of 
; 	either strarr, intarr, fltarr
;	is aware of field dividors including:
;	- the space, " "
;	max number of data lines is assumed. See var MAX.
;	rejects lines from an input db leading with a comment
;	symbol assumed to be of the set: # $ % & ;
;	
;	A. Muench
;	8/1999
;
;	USAGE: dbread,dir,file,dbtype,db,ncol,nrow,inmax=inmax,help=help
;	dir,file,dbtype are strings.
;	dir, file are dir and file locating the db to read in.
;	dbtype is string of the three forms:
;		dbtype = "STRING"
;		dbtype = "INTEGER"
;		dbtype = "FLOAT"
;	db,ncol,nrow are the returned values of the read in db.
;	inmax is an optional param for quickening dbread through 
;	shrinking the size of arrays.
;
;	Keyword "junklines" was added to allow for a specified number of
;	lines (junklines) at the beginning of a file to be skipped before
;	reading the information into "db".   tra 3/2001
;	
;	Exits cleanly with db=ncols=nrows=0 when empty array.  tra 7/13/2004
;	Last revised.   tra 7/13/2004  10:40AM

	pro dbread,dir,file,dbtype,db,ncol,nrow,inmax=inmax,help=help $
	,junklines=junklines
;
	on_error,0
;	on_ioerror,myioerr
	message,/reset
;
;       check for no params.
;
        IF n_params() EQ 0 OR keyword_set(help) THEN BEGIN
           print,'Usage: dbread,dir,file,dbtype,db,ncol,nrow,inmax'
	   print,'If inmax not set, max rows assumed to equal 32000'
	   print,'dbtype = "FLOAT","DOUBLE","STRING"
           RETURN
        ENDIF
;
;	set params
;
	IF keyword_set(inmax) THEN nmax = inmax ELSE nmax = 32000
;
	tarr = strarr(nmax)
	comments = ["#","$","%","&",";"]
	ncomment = n_elements(comments)
;
;	read in data file as strarr.
;	read in line by line as single strings.
;
	ifile = dir+file
;;; I commented out the following few lines because I don't have "FILE_EXISTS"  tra
;;;	IF (NOT FILE_EXIST(file,path=dir)) THEN BEGIN
;;;	   PRINT,"Sorry, input file to read, "+ifile+" does not exist."
;;;	   RETURN
;;;	ENDIF
;
	num = 0
	num = long(num)
	junk = ""
	openr,1,ifile
	if keyword_set(junklines) eq 1 then begin
	   for i=0,junklines-1 do begin
		readf,1,junk
	   endfor
	endif
	WHILE NOT EOF(1) DO BEGIN
;	print,"Working on line "+strtrim(num,2)
	   readf,1,junk & tarr[num] = junk
	   num = num + 1
	ENDWHILE
	close,1
;	
	emptyarray=0
	IF num EQ 0 THEN BEGIN
	  emptyarray=1
;	  message,'Empty Array',/IOERROR,/NONAME,/NOPREFIX
		PRINT,'EMPTY ARRAY'
		nrow = 0
	  	ncol = 0
		db   = 0
		message,/reset
           RETURN
	ENDIF
;
;	reformat array to accelerate computation
;	time lost by default large initial array.
;
	tarr = tarr[0:num-1]
	nrow = num
;
;	search for lines containing comments symbols
;	remove from array
;
	ccol = STRMID(tarr,0,1)
	j = 0
	FOR j = 0, ncomment-1 DO BEGIN
	    ngd = n_elements(tarr)
	    junk = WHERE(ccol EQ comments[j])
	    IF junk[0] NE -1 THEN BEGIN
	     good = WHERE(ccol NE comments[j],ngd)
	     IF ngd NE 0 THEN BEGIN
		tarr = tarr(good) 
		ccol = ccol(good)
	     ENDIF ELSE tarr=''
	    ENDIF 
	ENDFOR
;
;	recalculate the number of lines
;
	num  = ngd
	nrow = num
;
	emptyarray=0
	IF num EQ 0 THEN BEGIN
	  emptyarray=1
;	  message,'Empty Array',/IOERROR,/NONAME,/NOPREFIX
		PRINT,'EMPTY ARRAY'
		nrow = 0
	  	ncol = 0
		db   = 0
		message,/reset
           RETURN
	ENDIF
;
;	parse up the starr lines, use first row for
;	setting col designations.
;
	space = STR_SEP(tarr[0],' ',/TRIM)
	good  = WHERE(space NE '')
	space = space(good)
	ncol  = N_ELEMENTS(space)
	temp  = strarr(ncol,nrow)
	temp[0:ncol-1,0] = space[0:ncol-1]
;
;	work line by line.
;	check for seperators and transform into the str db
;
	i = 0
	i = LONG(i)
	FOR i = 1L,num-1 DO BEGIN
	    space = STR_SEP(tarr[i],' ',/TRIM)
	    good  = WHERE(space NE '')
	    space = space(good)
	    temp[0:ncol-1,i] = space[0:ncol-1]
; print,i
	ENDFOR
;
;	what is the type of output?
;
	IF dbtype EQ "STRING" THEN BEGIN
	   db = temp
	   RETURN
	ENDIF
	IF dbtype EQ "INTEGER" THEN BEGIN
	   db = ROUND(DOUBLE(temp))
	   RETURN
	ENDIF
	IF dbtype EQ "FLOAT" THEN BEGIN
	   db = FLOAT(DOUBLE(temp))
	   RETURN
	ENDIF
	IF dbtype EQ "DOUBLE" THEN BEGIN
	   db = DOUBLE(temp)
	   RETURN
	ENDIF
;
;       **********************************************************************
;       Error handler.
;
        myioerr : IF emptyarray EQ 1 THEN BEGIN
		PRINT,'EMPTY ARRAY'
		nrow = 0
	  	ncol = 0
		db   = 0
		message,/reset
           RETURN
           ENDIF 
;
	END
